--[[
This module should return a function that tells your story.
You can use the following functions to interface with the "terminal":

set_text(text, col, row)  -- sets the terminal's text at the given position to
                             the given text. Note that the text does not appear
                             immediately, but will be slowly revealed by the
                             trails.
                             `row` can range from 1 to `rows()`
                             `col` can range from 1 to `cols()`
                             anything outside this range will not be visible.

There are also functions that let you print in a more conventional way:
set_cursor(x, y)          -- sets the terminal cursor to the given position
add_text(text)            -- prints text, starting at the current cursor
                             position. When the function returns, the cursor
                             will be positioned right after the last letter
                             of this text.
new_line() -- increments the y position of the cursor, and resets the x position
              of the cursor to the x position that was set most recently, either
              via `set_cursor` or via `set_text`.

clear_text()              -- clears the entire screen
clear_line(line, to_line) -- clears one or more lines of text. If `to_line` is
                             given, then all lines from `line` through `to_line`
                             will be cleared. Otherwise only the text on line
                             `line` will be cleared.

wait(s)           -- waits `s` seconds before it returns
wait_for_key(tab) -- waits until the user presses a key, and returns the pressed
                     key as a string. If `tab` is `nil`, then the function will
                     return on any key press.
                     If you only care about certain keys, you can pass a table
                     containing those keys. The function will then only return
                     when one of those keys is pressed.
                     For example, if you want to check for a, b and c, you could
                     call `wait_for_key({"a", "b", "c"})`.
                     For a list of valid keys, check the "Key string" column on
                     https://love2d.org/wiki/KeyConstant

rows(), cols() -- returns the number of rows and columns in the terminal. This
                  value can change when the window size changes.

quit() -- fades out the trails and text, and then closes the terminal. To quit
          immediately, use `love.events.push("quit")`.

config -- a table containing various values that define the terminal look.
          Some terminology first:
           - trails are the strings that move from the top to the bottom of the
             terminal
           - static text is the text that is set through functions like set_text
             and add_text

          The following values can be changed from within the story:

          background_color     -- the terminal's background color
          font_color           -- the color of letters in the code trails
          font_color_highlight -- the color of static letters, as well as the
                                  first letter of every trail

          speed -- the speed with which the code trails move down, in lines
                   per second.
          min_trail_length -- the minimum length of code trails
          max_trail_length -- the maximum length of code trails
          trails_per_s -- the number of trails that will be spawned, per second
          fade -- determines how quickly trails fade out. Higher values cause
                  the trails to fade out faster.
          chance_mutate -- the chance for a character in the code trail to
                           mutate to a different character.

          static_glitch_chance -- the chance per frame that a static letter
                                  glitches and temporarily displays a random
                                  letter instead.
          static_glitch_duration -- how long a static letter stays glitched, in
                                    seconds
          static_max_afterlife -- maximum time in seconds until a static letter
                                  disappears after the corresponding text was
                                  cleared

You can of course also use any other functions that are available in lua or love.
]]

-- Replace the content of this function with your own story
return function()
  local cy = math.floor(rows() / 2)
  local cx = math.floor(cols() / 2)

  wait(10)
  set_text("WELCOME, [USER #31CF73 | OBEY_LOVE]", cx - 17, cy)
  set_text("PRESS (RETURN) TO CONNECT TO MAINFRAME 4", cx - 20, cy + 2)
  wait_for_key({"return"})

  clear_text()
  wait(1)
  set_cursor(3, 2)
  add_text("LOADING PLATFORM MODULE...    ")
  wait(4)
  add_text("  DONE\n")
  add_text("LOADING MAINFRAME PROTOCOLS...")
  wait(4)
  add_text("  DONE\n")
  add_text("SYNCHRONIZING UPLINK...       ")
  wait(12)
  add_text(" ERROR\n\n")

  add_text
  [[COULD NOT CONNECT TO MAINFRAME 4]]
  wait(10)

  set_text("YOU HAVE ONE (1) NEW MESSAGE FROM [USER #341A9E | STALENHAG, SIMON]", 3, rows() - 3)
  new_line()
  add_text("PRESS (R) TO READ IT.")
  wait_for_key({"r"})

  clear_text()
  set_cursor(3, 2)
  add_text(
[[
MESSAGE  #9FF7512A6
FROM     [USER #341A9E | STALENHAG, SIMON]
TO       [USER #31CF73 | OBEY_LOVE]
RECEIVED 1988-11-23 08:42:13 GMT+1

Hi, this is Simon from signal processing.
Last night, the sensors in the southern tunnel picked up some weird vibrations,
unlike anything I've seen since I got here. I was hoping that you could take a
look at it? I heard you built some tools for this kind of analysis while you
spent time on the surface.
The raw readings are in the attachment.
Cheers,
S.
]])

  wait(10)
  new_line()
  add_text(
[[
THERE IS ONE (1) ATTACHMENT
  weird_stuff.asd (12KB)
WOULD YOU LIKE TO OPEN IT? (Y/N)
]])
  local key = wait_for_key({"y", "n"})
  if key == "y" then
    config.background_color = {14, 7, 8}
    config.font_color = {254, 77, 79}
    config.font_color_highlight = {252, 171, 171}
    config.speed = 16

    clear_text()
    set_text("INFECTION DETECTED", 3, 2)
    new_line()
    add_text("SYSTEM LOCK-DOWN INITIALIZED")
  end

  wait(10)
  clear_text()
  set_text("END OF MEMORY DUMP", cx - 9, rows() - 3)
  set_text("PRESS (Q) TO QUIT", cx - 8, rows() - 2)
  wait_for_key({"q"})
  quit()

end

